@@ -6,7 +6,7 @@ var path = require('path')
 
 var BrowserslistError = require('./error')
 var env = require('./node')
-var parse = require('./parse') // Will load browser.js in webpack
+var parseWithoutCache = require('./parse') // Will load browser.js in webpack
 
 var YEAR = 365.259641 * 24 * 60 * 60 * 1000
 var ANDROID_EVERGREEN_FIRST = '37'
@@ -319,7 +319,7 @@ function isSupported(flags, withPartial) {
 }
 
 function resolve(queries, context) {
-  return parse(QUERIES, queries).reduce(function (result, node, index) {
+  return parseQueries(queries).reduce(function (result, node, index) {
     if (node.not && index === 0) {
       throw new BrowserslistError(
         'Write any browsers query (for instance, `defaults`) ' +
@@ -395,19 +395,26 @@ function checkQueries(queries) {
 }
 
 var cache = {}
+var parseCache = {}
 
 function browserslist(queries, opts) {
   opts = prepareOpts(opts)
   queries = prepareQueries(queries, opts)
   checkQueries(queries)
 
+  var needsPath = parseQueries(queries).some(function (node) {
+    return QUERIES[node.type].needsPath
+  })
   var context = {
     ignoreUnknownVersions: opts.ignoreUnknownVersions,
     dangerousExtend: opts.dangerousExtend,
     mobileToDesktop: opts.mobileToDesktop,
-    path: opts.path,
     env: opts.env
   }
+  // Removing to avoid using context.path without marking query as needsPath
+  if (needsPath) {
+    context.path = opts.path
+  }
 
   env.oldDataWarning(browserslist.data)
   var stats = env.getStat(opts, browserslist.data)
@@ -441,11 +448,35 @@ function browserslist(queries, opts) {
   return result
 }
 
+function parseQueries(queries) {
+  var cacheKey = JSON.stringify(queries)
+  if (cacheKey in parseCache) return parseCache[cacheKey]
+  var result = parseWithoutCache(QUERIES, queries)
+  if (!env.env.BROWSERSLIST_DISABLE_CACHE) {
+    parseCache[cacheKey] = result
+  }
+  return result
+}
+
+function loadCustomUsage(context, config) {
+  var stats = env.loadStat(context, config, browserslist.data)
+  if (stats) {
+    context.customUsage = {}
+    for (var browser in stats) {
+      fillUsage(context.customUsage, browser, stats[browser])
+    }
+  }
+  if (!context.customUsage) {
+    throw new BrowserslistError('Custom usage statistics was not provided')
+  }
+  return context.customUsage
+}
+
 browserslist.parse = function (queries, opts) {
   opts = prepareOpts(opts)
   queries = prepareQueries(queries, opts)
   checkQueries(queries)
-  return parse(QUERIES, queries)
+  return parseQueries(queries)
 }
 
 // Will be filled by Can I Use data below
@@ -581,19 +612,21 @@ function coverQuery(context, node) {
       env.loadCountry(browserslist.usage, place, browserslist.data)
       usage = browserslist.usage[place]
     }
+  } else if (node.config) {
+    usage = loadCustomUsage(context, node.config)
   }
   var versions = Object.keys(usage).sort(function (a, b) {
     return usage[b] - usage[a]
   })
-  var coveraged = 0
+  var covered = 0
   var result = []
   var version
   for (var i = 0; i < versions.length; i++) {
     version = versions[i]
     if (usage[version] === 0) break
-    coveraged += usage[version]
+    covered += usage[version]
     result.push(version)
-    if (coveraged >= coverage) break
+    if (covered >= coverage) break
   }
   return result
 }
@@ -812,17 +845,7 @@ var QUERIES = {
     regexp: /^(>=?|<=?)\s*(\d+|\d+\.\d+|\.\d+)%\s+in\s+(\S+)\s+stats$/,
     select: function (context, node) {
       var popularity = parseFloat(node.popularity)
-      var stats = env.loadStat(context, node.config, browserslist.data)
-      if (stats) {
-        context.customUsage = {}
-        for (var browser in stats) {
-          fillUsage(context.customUsage, browser, stats[browser])
-        }
-      }
-      if (!context.customUsage) {
-        throw new BrowserslistError('Custom usage statistics was not provided')
-      }
-      var usage = context.customUsage
+      var usage = loadCustomUsage(context, node.config)
       return Object.keys(usage).reduce(function (result, version) {
         var percentage = usage[version]
         if (percentage == null) {
@@ -896,6 +919,11 @@ var QUERIES = {
     regexp: /^cover\s+(\d+|\d+\.\d+|\.\d+)%\s+in\s+(my\s+stats|(alt-)?\w\w)$/i,
     select: coverQuery
   },
+  cover_config: {
+    matches: ['coverage', 'config'],
+    regexp: /^cover\s+(\d+|\d+\.\d+|\.\d+)%\s+in\s+(\S+)\s+stats$/i,
+    select: coverQuery
+  },
   supports: {
     matches: ['supportType', 'feature'],
     regexp: /^(?:(fully|partially)\s+)?supports\s+([\w-]+)$/,
@@ -1133,6 +1161,7 @@ var QUERIES = {
   browserslist_config: {
     matches: [],
     regexp: /^browserslist config$/i,
+    needsPath: true,
     select: function (context) {
       return browserslist(undefined, context)
     }
@@ -1140,6 +1169,7 @@ var QUERIES = {
   extends: {
     matches: ['config'],
     regexp: /^extends (.+)$/i,
+    needsPath: true,
     select: function (context, node) {
       return resolve(env.loadQueries(context, node.config), context)
     }
